# Python imports
from urlparse import urlparse

# Local imports
from funky import first_true


def is_http_url(url, parsed):
    if parsed.scheme in ('http', 'https'):
        return parsed.scheme
    return None


def is_git_url(url, parsed):
    if parsed.scheme == 'git':
        return parsed.scheme
    return None


def is_ssh_url(url, parsed):
    if parsed.scheme == 'git+ssh':
        return parsed.scheme
    return None


def get_protocol(url):
    schemers = [
        is_git_url,
        is_ssh_url,
        is_http_url,
    ]

    parsed = urlparse(url)

    try:
        return first_true([
            schemer(url, parsed)
            for schemer in schemers
        ])
    except:
        pass
    return None


def get_password(url):
    pass


def get_user(url):
    pass


def parse_url(url, defaults=None):
    """Parse a url corresponding to a git repository
    """
    DEFAULTS = {
        'protocol': 'git+ssh',
    }
    defaults = defaults or DEFAULTS

    protocol = get_protocol() or defaults.get('protocol')

    return {
        'domain': domain,
        'protocol': protocol,
        'user': user,
        'password': password,
        'path': path,
    }




# Python imports
import os

# Dulwich imports
from dulwich.server import FileSystemBackend, TCPGitServer, UploadPackHandler, ReceivePackHandler

# Dict entries
WRITE = (('git-upload-pack', UploadPackHandler),)
READ = (('git-receive-pack', ReceivePackHandler),)

READ_HANDLERS = dict(READ)

WRITE_HANDLERS = dict(WRITE)

READ_WRITE_HANDLERS = dict(READ + WRITE)

PERM_MAPPING = {
   'r': READ_HANDLERS,
   'w': WRITE_HANDLERS,
   'rw': READ_WRITE_HANDLERS,
   'wr': READ_WRITE_HANDLERS,
}


class SubFileSystemBackend(FileSystemBackend):
    """A simple FileSystemBackend restricted to a given path
    """
    def __init__(self, root_path):
        self.root_path = root_path

    def rewrite_path(self, path):
        return os.path.join(self.root_path, path)

    def open_repository(self, path):
        stripped_path = path.strip('/')
        full_path = self.rewrite_path(stripped_path)

        print('opening %s' % path)
        print('full path = %s' % full_path)

        return super(SubFileSystemBackend, self).open_repository(full_path)


class GitServer(TCPGitServer):
    """Server using the git protocol over TCP
    """
    def __init__(self, root_path=None, listen_addr=None, perm=None, *args, **kwargs):
        # Default values
        self.perm = perm or 'r'
        self.root_path = root_path or '/'
        self.listen_addr = listen_addr or 'localhost'

        # Backend
        backend = SubFileSystemBackend(self.root_path)

        # Handlers by permissions
        handlers = PERM_MAPPING.get(self.perm, READ_HANDLERS)

        # This is ugly and due to the fact that TCPGitServer is and old style class
        TCPGitServer.__init__(self, backend, self.listen_addr, handlers=handlers, *args, **kwargs)



#!/usr/bin/python

"""
Setup script for Gittle.
"""

import platform
windows = platform.system() == 'Windows'
try:
    from setuptools import setup
except ImportError:
    has_setuptools = False
    from distutils.core import setup
else:
    has_setuptools = True

version_string = '0.3.0'

setup_kwargs = {
    'name': 'gittle',
    'description': 'A high level pure python git implementation',
    'keywords': 'git dulwich pure python gittle',
    'version': version_string,
    'url': 'https://github.com/FriendCode/gittle',
    'license': 'MIT',
    'author': "Aaron O'Mullan",
    'author_email': 'aaron@friendco.de',
    'long_description': """
    Gittle is a wrapper around dulwich. It provides an easy and familiar interface to git.
    It's pure python (no dependancy on the git binary) and has no other dependancies besides
    the python stdlib, dulwich and paramiko (optional).
    """,
    'packages': ['gittle', 'gittle.utils'],
    'install_requires': [
    # PyPI
    'paramiko==1.10.0',
    'pycrypto==2.6',

    # Non PyPI
    'dulwich',
    'funky',
    'mimer',
    ],
    'dependency_links': [
        'https://github.com/AaronO/dulwich/tarball/eebb032b2b7b982d21d636ac50b6e45de58b208b#egg=dulwich-0.9.1-2',
        'https://github.com/FriendCode/funky/tarball/e89cb2ce4374bf2069c7f669e52e046f63757241#egg=funky-0.0.1',
        'https://github.com/FriendCode/mimer/tarball/a812e5f631b9b5c969df5a2ea84b635490a96ced#egg=mimer-0.0.1',
    ],
}


try:

    # Run setup with C extensions
    setup(**setup_kwargs)

except SystemExit as exc:

    import logging
    logging.exception(exc)
    logging.info("retrying installation without VisualStudio...")

    # Remove C dependencies
    install_requires = [r for r in setup_kwargs['install_requires']
                        if r.split('=')[0] not in ('paramiko', 'pycrypto')]

    # Install dulwich as pure Python
    if windows and has_setuptools:
        from setuptools.command.easy_install import easy_install
        run_setup = easy_install.run_setup

        def _run_setup(self, setup_script, setup_base, args):
            """Alternate run_setup function to pass '--pure' to the
            Dulwich installer on Windows.
            """
            if 'dulwich' in setup_script:
                args.insert(0, '--pure')
            run_setup(self, setup_script, setup_base, args)

        easy_install.run_setup = _run_setup

    # Run setup without C extensions
    setup_kwargs['install_requires'] = install_requires
    setup(**setup_kwargs)
